package Apache::LogSTDERR;

use strict;
use Apache::LogSTDERR::Config ();
use Apache::Constants qw(OK DECLINED);

BEGIN {
    no strict;
    $VERSION = '0.12';
    @ISA = qw(DynaLoader);
    __PACKAGE__->bootstrap($VERSION);
}

sub PRINTF {
    my $self = shift;
    my $fmt = shift;
    $self->PRINT(sprintf $fmt, @_);
}

sub WRITE {
    my($self, $buff, $length, $offset) = @_;
    return unless($buff && $offset && $length);
    my $send = substr($buff, $offset, $length);
    $self->PRINT($send);
}

1; 
__END__

=head1 NAME

Apache::LogSTDERR - Buffered logging to STDERR

=head1 SYNOPSIS

  # enable STDERR buffering
  PerlModule      Apache::LogSTDERR
  PerlInitHandler Apache::LogSTDERR

  # redirect stderr
  HookStderr      |logger -p local3.debug

  # redirect STDERR
  HookPerlStderr  |logger -p local3.debug

=head1 DESCRIPTION

This module ties C<STDERR> to a buffer which is flushed at the end of each
request. Also it allows C<stderr> and C<STDERR> to be redirected to a file
or a piped log.

=head1 USAGE

Enable STDERR buffering simply by configuring the module as a
B<PerlInitHandler> in your httpd configuration file:

  PerlModule      Apache::LogSTDERR
  PerlInitHandler Apache::LogSTDERR

When the module is loaded, a buffer is created and a cleanup is registered.
During request processing, data written to C<STDERR> is placed into the
buffer. When the request is cleaned up, the buffer is flushed to the
B<HookPerlStderr> (see below) or the B<ErrorLog>.

=head1 DIRECTIVES

=over 4

=item HookStderr

When the Apache builtin syslog support is used, the C<stderr> stream is
redirected to C</dev/null>. This means Perl warnings, any messages from
C<die(>), C<croak()>, etc. will also end up in the black hole. I<HookStderr>
redirects C<stderr> to a file or piped log.

Examples:

  HookStderr # defaults to logs/stderr_log
  HookStderr /tmp/stderr.log
  HookStderr |logger -p local3.debug

Output to C<stderr> is not buffered by this module or otherwise related to
C<STDERR> in any way. This is simply a utility that was stuffed into the
module for convenience.

=item HookPerlStderr

Normally C<STDERR> output is written to the B<ErrorLog>. Sometimes, however,
it is useful to redirect C<STDERR> to a different location. For instance,
you might want Apache errors to go to syslog, which you monitor continuously,
but your application debug messages to be written into a file that you check
once a day. I<HookPerlStderr> redirects C<STDERR> to a file or piped log.

Examples:

  HookPerlStderr # defaults to logs/STDERR_log
  HookPerlStderr /tmp/STDERR.log
  HookPerlStderr |logger -p local3.debug

=back

=head1 AUTHOR

Doug MacEachern, <dougm@pobox.com>
